'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createCURD;

var _location = require('./location');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * 创建方法
 * @param method
 * @param types
 * @param prefix
 * @returns {Function}
 */
function createMenthod(method, types, prefix) {
  return function () {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        params = _ref.params,
        data = _ref.data;

    return {
      types: types,
      promise: function promise(client) {
        return client[method](prefix, { params: params, data: data });
      }
    };
  };
}
/**
 * 创建方法和常量
 * @param prefix  前缀
 * @param actions
 * @param action
 * @param constants
 * @param methods
 * @param name
 * @param pageName
 */
function createMenthodsAndConstants(prefix, actions, action, constants, methods, name, pageName) {
  pageName = pageName ? '-' + pageName : '';

  if (~actions.indexOf(action)) {
    var _Object$assign;

    var types = ['' + prefix + pageName + '/' + name, '' + prefix + pageName + '/' + name + '_SUCCESS', '' + prefix + pageName + '/' + name + '_FAIL', '' + prefix + pageName + '/' + name + '_CLEAR'];
    // 常量
    Object.assign(constants, (_Object$assign = {}, _defineProperty(_Object$assign, name, types[0]), _defineProperty(_Object$assign, name + '_SUCCESS', types[1]), _defineProperty(_Object$assign, name + '_FAIL', types[2]), _defineProperty(_Object$assign, name + '_CLEAR', types[3]), _Object$assign));
    // 方法
    if (action === 'C') {
      methods.create = createMenthod('post', types, prefix);
    } else if (action === 'U') {
      methods.update = createMenthod('put', types, prefix);
    } else if (action === 'R') {
      methods.load = createMenthod('get', types, prefix);
    } else if (action === 'D') {
      methods.del = createMenthod('del', types, prefix);
    }
  }
}
/**
 *
 * @param prefix 前缀
 * @param actions
 * @param pageName
 * @returns {{methods: {}, createReducer: createReducer}}
 */
function createCURD(prefix, actions, pageName) {
  var constants = {};
  var methods = {};
  var actionsMap = {
    'C': 'CREATE',
    'U': 'UPDATE',
    'R': 'LOAD',
    'D': 'DELETE'
  };

  actions.toUpperCase().split('').forEach(function (action) {
    createMenthodsAndConstants(prefix, actions, action, constants, methods, actionsMap[action], pageName);
  });

  var createReducer = function createReducer(state, action) {
    switch (action.type) {
      //加载数据中
      case constants.LOAD:
      case constants.CREATE:
      case constants.UPDATE:
      case constants.DELETE:
        return _extends({}, state, {
          loading: true,
          loaded: false
        });
      //加载成功
      case constants.LOAD_SUCCESS:
      case constants.CREATE_SUCCESS:
      case constants.UPDATE_SUCCESS:
      case constants.DELETE_SUCCESS:
        return _extends({}, state, {
          loading: false,
          loaded: true,
          data: action.result
        });
      //加载失败
      case constants.LOAD_FAIL:
      case constants.CREATE_FAIL:
      case constants.UPDATE_FAIL:
      case constants.DELETE_FAIL:
        return _extends({}, state, {
          loading: false,
          loaded: false,
          error: action.error
        });
      // 解决跳出页面 数据还在的问题
      case _location.LOCATION_CHANGE:
        return {};
    }
  };
  return { methods: methods, createReducer: createReducer };
}